"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f12df4b2ce43

from .basesdk import BaseSDK
import httpx
from mistralai.client import errors, models, utils
from mistralai.client._hooks import HookContext
from mistralai.client.types import OptionalNullable, UNSET
from mistralai.client.utils import get_security_from_env
from mistralai.client.utils.unmarshal_json_response import unmarshal_json_response
from typing import List, Mapping, Optional, Union


class Files(BaseSDK):
    r"""Files API"""

    def upload(
        self,
        *,
        file: Union[models.File, models.FileTypedDict],
        expiry: OptionalNullable[int] = UNSET,
        visibility: Optional[
            models.FilesAPIRoutesUploadFileFileVisibility
        ] = "workspace",
        purpose: Optional[models.FilePurpose] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateFileResponse:
        r"""Upload File

        Upload a file that can be used across various endpoints.

        The size of individual files can be a maximum of 512 MB. The Fine-tuning API only supports .jsonl files.

        Please contact us if you need to increase these storage limits.

        :param file: The File object (not file name) to be uploaded.
            To upload a file and specify a custom file name you should format your request as such:
            ```bash
            file=@path/to/your/file.jsonl;filename=custom_name.jsonl
            ```
            Otherwise, you can just keep the original file name:
            ```bash
            file=@path/to/your/file.jsonl
            ```
        :param expiry:
        :param visibility:
        :param purpose:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MultiPartBodyParams(
            expiry=expiry,
            visibility=visibility,
            purpose=purpose,
            file=utils.get_pydantic_model(file, models.File),
        )

        req = self._build_request(
            method="POST",
            path="/v1/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.MultiPartBodyParams
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_upload_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CreateFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def upload_async(
        self,
        *,
        file: Union[models.File, models.FileTypedDict],
        expiry: OptionalNullable[int] = UNSET,
        visibility: Optional[
            models.FilesAPIRoutesUploadFileFileVisibility
        ] = "workspace",
        purpose: Optional[models.FilePurpose] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateFileResponse:
        r"""Upload File

        Upload a file that can be used across various endpoints.

        The size of individual files can be a maximum of 512 MB. The Fine-tuning API only supports .jsonl files.

        Please contact us if you need to increase these storage limits.

        :param file: The File object (not file name) to be uploaded.
            To upload a file and specify a custom file name you should format your request as such:
            ```bash
            file=@path/to/your/file.jsonl;filename=custom_name.jsonl
            ```
            Otherwise, you can just keep the original file name:
            ```bash
            file=@path/to/your/file.jsonl
            ```
        :param expiry:
        :param visibility:
        :param purpose:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MultiPartBodyParams(
            expiry=expiry,
            visibility=visibility,
            purpose=purpose,
            file=utils.get_pydantic_model(file, models.File),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.MultiPartBodyParams
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_upload_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CreateFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def list(
        self,
        *,
        page: Optional[int] = 0,
        page_size: Optional[int] = 100,
        include_total: Optional[bool] = True,
        sample_type: OptionalNullable[List[models.SampleType]] = UNSET,
        source: OptionalNullable[List[models.Source]] = UNSET,
        search: OptionalNullable[str] = UNSET,
        purpose: OptionalNullable[models.FilePurpose] = UNSET,
        mimetypes: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListFilesResponse:
        r"""List Files

        Returns a list of files that belong to the user's organization.

        :param page:
        :param page_size:
        :param include_total:
        :param sample_type:
        :param source:
        :param search:
        :param purpose:
        :param mimetypes:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesListFilesRequest(
            page=page,
            page_size=page_size,
            include_total=include_total,
            sample_type=sample_type,
            source=source,
            search=search,
            purpose=purpose,
            mimetypes=mimetypes,
        )

        req = self._build_request(
            method="GET",
            path="/v1/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_list_files",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListFilesResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page: Optional[int] = 0,
        page_size: Optional[int] = 100,
        include_total: Optional[bool] = True,
        sample_type: OptionalNullable[List[models.SampleType]] = UNSET,
        source: OptionalNullable[List[models.Source]] = UNSET,
        search: OptionalNullable[str] = UNSET,
        purpose: OptionalNullable[models.FilePurpose] = UNSET,
        mimetypes: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListFilesResponse:
        r"""List Files

        Returns a list of files that belong to the user's organization.

        :param page:
        :param page_size:
        :param include_total:
        :param sample_type:
        :param source:
        :param search:
        :param purpose:
        :param mimetypes:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesListFilesRequest(
            page=page,
            page_size=page_size,
            include_total=include_total,
            sample_type=sample_type,
            source=source,
            search=search,
            purpose=purpose,
            mimetypes=mimetypes,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_list_files",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListFilesResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def retrieve(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetFileResponse:
        r"""Retrieve File

        Returns information about a specific file.

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesRetrieveFileRequest(
            file_id=file_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/files/{file_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_retrieve_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GetFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def retrieve_async(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetFileResponse:
        r"""Retrieve File

        Returns information about a specific file.

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesRetrieveFileRequest(
            file_id=file_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/files/{file_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_retrieve_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GetFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DeleteFileResponse:
        r"""Delete File

        Delete a file.

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesDeleteFileRequest(
            file_id=file_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/files/{file_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_delete_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DeleteFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DeleteFileResponse:
        r"""Delete File

        Delete a file.

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesDeleteFileRequest(
            file_id=file_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/files/{file_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_delete_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DeleteFileResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def download(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> httpx.Response:
        r"""Download File

        Download a file

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesDownloadFileRequest(
            file_id=file_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/files/{file_id}/content",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/octet-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_download_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/octet-stream"):
            return http_res
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.SDKError("Unexpected response received", http_res, http_res_text)

    async def download_async(
        self,
        *,
        file_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> httpx.Response:
        r"""Download File

        Download a file

        :param file_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesDownloadFileRequest(
            file_id=file_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/files/{file_id}/content",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/octet-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_download_file",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/octet-stream"):
            return http_res
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.SDKError("Unexpected response received", http_res, http_res_text)

    def get_signed_url(
        self,
        *,
        file_id: str,
        expiry: Optional[int] = 24,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetSignedURLResponse:
        r"""Get Signed Url

        :param file_id:
        :param expiry: Number of hours before the url becomes invalid. Defaults to 24h
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesGetSignedURLRequest(
            file_id=file_id,
            expiry=expiry,
        )

        req = self._build_request(
            method="GET",
            path="/v1/files/{file_id}/url",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_get_signed_url",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GetSignedURLResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def get_signed_url_async(
        self,
        *,
        file_id: str,
        expiry: Optional[int] = 24,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetSignedURLResponse:
        r"""Get Signed Url

        :param file_id:
        :param expiry: Number of hours before the url becomes invalid. Defaults to 24h
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FilesAPIRoutesGetSignedURLRequest(
            file_id=file_id,
            expiry=expiry,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/files/{file_id}/url",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="files_api_routes_get_signed_url",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GetSignedURLResponse, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 60a51ff1682b

from __future__ import annotations
from .agenthandoffentry import AgentHandoffEntry, AgentHandoffEntryTypedDict
from .functioncallentry import FunctionCallEntry, FunctionCallEntryTypedDict
from .functionresultentry import FunctionResultEntry, FunctionResultEntryTypedDict
from .messageinputentry import MessageInputEntry, MessageInputEntryTypedDict
from .messageoutputentry import MessageOutputEntry, MessageOutputEntryTypedDict
from .toolexecutionentry import ToolExecutionEntry, ToolExecutionEntryTypedDict
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


EntryTypedDict = TypeAliasType(
    "EntryTypedDict",
    Union[
        FunctionResultEntryTypedDict,
        MessageInputEntryTypedDict,
        MessageOutputEntryTypedDict,
        AgentHandoffEntryTypedDict,
        ToolExecutionEntryTypedDict,
        FunctionCallEntryTypedDict,
    ],
)


Entry = TypeAliasType(
    "Entry",
    Union[
        FunctionResultEntry,
        MessageInputEntry,
        MessageOutputEntry,
        AgentHandoffEntry,
        ToolExecutionEntry,
        FunctionCallEntry,
    ],
)


class ConversationHistoryTypedDict(TypedDict):
    r"""Retrieve all entries in a conversation."""

    conversation_id: str
    entries: List[EntryTypedDict]
    object: Literal["conversation.history"]


class ConversationHistory(BaseModel):
    r"""Retrieve all entries in a conversation."""

    conversation_id: str

    entries: List[Entry]

    object: Annotated[
        Annotated[
            Optional[Literal["conversation.history"]],
            AfterValidator(validate_const("conversation.history")),
        ],
        pydantic.Field(alias="object"),
    ] = "conversation.history"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["object"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ConversationHistory.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 3d1cd35fecc6

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class DeleteCampaignV1ObservabilityCampaignsCampaignIDDeleteRequestTypedDict(TypedDict):
    campaign_id: str


class DeleteCampaignV1ObservabilityCampaignsCampaignIDDeleteRequest(BaseModel):
    campaign_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f96acbcd45f7

from __future__ import annotations
from .feedresultchatcompletioneventpreview import (
    FeedResultChatCompletionEventPreview,
    FeedResultChatCompletionEventPreviewTypedDict,
)
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class SearchChatCompletionEventsResponseTypedDict(TypedDict):
    completion_events: FeedResultChatCompletionEventPreviewTypedDict


class SearchChatCompletionEventsResponse(BaseModel):
    completion_events: FeedResultChatCompletionEventPreview



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .contentchunk import ContentChunk, ContentChunkTypedDict
from mistralai.gcp.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.gcp.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ToolMessageContentTypedDict = TypeAliasType(
    "ToolMessageContentTypedDict", Union[str, List[ContentChunkTypedDict]]
)


ToolMessageContent = TypeAliasType("ToolMessageContent", Union[str, List[ContentChunk]])


class ToolMessageTypedDict(TypedDict):
    content: Nullable[ToolMessageContentTypedDict]
    role: Literal["tool"]
    tool_call_id: NotRequired[Nullable[str]]
    name: NotRequired[Nullable[str]]


class ToolMessage(BaseModel):
    content: Nullable[ToolMessageContent]

    role: Annotated[
        Annotated[Literal["tool"], AfterValidator(validate_const("tool"))],
        pydantic.Field(alias="role"),
    ] = "tool"

    tool_call_id: OptionalNullable[str] = UNSET

    name: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["tool_call_id", "name"])
        nullable_fields = set(["content", "tool_call_id", "name"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ToolMessage.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 969acd9d6220

from __future__ import annotations
from .tempotraceresource import TempoTraceResource, TempoTraceResourceTypedDict
from .tempotracescopespan import TempoTraceScopeSpan, TempoTraceScopeSpanTypedDict
from mistralai.client.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TempoTraceBatchTypedDict(TypedDict):
    resource: TempoTraceResourceTypedDict
    scope_spans: NotRequired[List[TempoTraceScopeSpanTypedDict]]
    r"""The spans of the scope"""


class TempoTraceBatch(BaseModel):
    resource: TempoTraceResource

    scope_spans: Annotated[
        Optional[List[TempoTraceScopeSpan]], pydantic.Field(alias="scopeSpans")
    ] = None
    r"""The spans of the scope"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["scopeSpans"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TempoTraceBatch.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1b7d73eddf51

from __future__ import annotations
from .metadatadict import MetadataDict, MetadataDictTypedDict
from datetime import datetime
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


AgentConversationAgentVersionTypedDict = TypeAliasType(
    "AgentConversationAgentVersionTypedDict", Union[str, int]
)


AgentConversationAgentVersion = TypeAliasType(
    "AgentConversationAgentVersion", Union[str, int]
)


class AgentConversationTypedDict(TypedDict):
    id: str
    created_at: datetime
    updated_at: datetime
    agent_id: str
    name: NotRequired[Nullable[str]]
    r"""Name given to the conversation."""
    description: NotRequired[Nullable[str]]
    r"""Description of the what the conversation is about."""
    metadata: NotRequired[Nullable[MetadataDictTypedDict]]
    r"""Custom metadata for the conversation."""
    object: Literal["conversation"]
    agent_version: NotRequired[Nullable[AgentConversationAgentVersionTypedDict]]


class AgentConversation(BaseModel):
    id: str

    created_at: datetime

    updated_at: datetime

    agent_id: str

    name: OptionalNullable[str] = UNSET
    r"""Name given to the conversation."""

    description: OptionalNullable[str] = UNSET
    r"""Description of the what the conversation is about."""

    metadata: OptionalNullable[MetadataDict] = UNSET
    r"""Custom metadata for the conversation."""

    object: Annotated[
        Annotated[
            Optional[Literal["conversation"]],
            AfterValidator(validate_const("conversation")),
        ],
        pydantic.Field(alias="object"),
    ] = "conversation"

    agent_version: OptionalNullable[AgentConversationAgentVersion] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["name", "description", "metadata", "object", "agent_version"]
        )
        nullable_fields = set(["name", "description", "metadata", "agent_version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    AgentConversation.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 584e0a98082a

from __future__ import annotations
from .eventtype import EventType
from .workflowexecutiontracesummaryattributesvalues import (
    WorkflowExecutionTraceSummaryAttributesValues,
    WorkflowExecutionTraceSummaryAttributesValuesTypedDict,
)
from mistralai.client.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class WorkflowExecutionTraceEventTypedDict(TypedDict):
    name: str
    r"""Name of the event"""
    id: str
    r"""The ID of the event"""
    timestamp_unix_nano: int
    r"""The timestamp of the event in nanoseconds since the Unix epoch"""
    attributes: Dict[
        str, Nullable[WorkflowExecutionTraceSummaryAttributesValuesTypedDict]
    ]
    r"""The attributes of the event"""
    type: NotRequired[EventType]
    internal: NotRequired[bool]
    r"""Whether the event is internal"""


class WorkflowExecutionTraceEvent(BaseModel):
    name: str
    r"""Name of the event"""

    id: str
    r"""The ID of the event"""

    timestamp_unix_nano: int
    r"""The timestamp of the event in nanoseconds since the Unix epoch"""

    attributes: Dict[str, Nullable[WorkflowExecutionTraceSummaryAttributesValues]]
    r"""The attributes of the event"""

    type: Optional[EventType] = None

    internal: Optional[bool] = False
    r"""Whether the event is internal"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type", "internal"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b51ec296cc92

from __future__ import annotations
from typing import Literal


EncodingFormat = Literal[
    "float",
    "base64",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b4d3fb07196e

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class AuthDataTypedDict(TypedDict):
    client_id: str
    client_secret: str


class AuthData(BaseModel):
    client_id: str

    client_secret: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 9455e38a8c31

from __future__ import annotations
from .conversationpayload import ConversationPayload, ConversationPayloadTypedDict
from mistralai.client.types import BaseModel
from typing import Any, Dict
from typing_extensions import TypedDict


class CreateDatasetRecordRequestTypedDict(TypedDict):
    payload: ConversationPayloadTypedDict
    properties: Dict[str, Any]


class CreateDatasetRecordRequest(BaseModel):
    payload: ConversationPayload

    properties: Dict[str, Any]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ba62f90873c5

from __future__ import annotations
from mistralai.client.types import BaseModel, UnrecognizedStr
from typing import Any, Literal, Union
from typing_extensions import TypedDict


Op = Union[
    Literal[
        "lt",
        "lte",
        "gt",
        "gte",
        "startswith",
        "istartswith",
        "endswith",
        "iendswith",
        "contains",
        "icontains",
        "matches",
        "notcontains",
        "inotcontains",
        "eq",
        "neq",
        "isnull",
        "includes",
        "excludes",
        "len_eq",
    ],
    UnrecognizedStr,
]


class FilterConditionTypedDict(TypedDict):
    field: str
    op: Op
    value: Any


class FilterCondition(BaseModel):
    field: str

    op: Op

    value: Any



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2e9ef5800117

from __future__ import annotations
from .function import Function, FunctionTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class FunctionToolTypedDict(TypedDict):
    function: FunctionTypedDict
    type: Literal["function"]


class FunctionTool(BaseModel):
    function: Function

    type: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"


try:
    FunctionTool.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 5bd25cdf9c7a

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing import Any, Dict, Union
from typing_extensions import TypeAliasType, TypedDict


RealtimeTranscriptionErrorDetailMessageTypedDict = TypeAliasType(
    "RealtimeTranscriptionErrorDetailMessageTypedDict", Union[str, Dict[str, Any]]
)
r"""Human-readable error message."""


RealtimeTranscriptionErrorDetailMessage = TypeAliasType(
    "RealtimeTranscriptionErrorDetailMessage", Union[str, Dict[str, Any]]
)
r"""Human-readable error message."""


class RealtimeTranscriptionErrorDetailTypedDict(TypedDict):
    message: RealtimeTranscriptionErrorDetailMessageTypedDict
    r"""Human-readable error message."""
    code: int
    r"""Internal error code for debugging."""


class RealtimeTranscriptionErrorDetail(BaseModel):
    message: RealtimeTranscriptionErrorDetailMessage
    r"""Human-readable error message."""

    code: int
    r"""Internal error code for debugging."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6fefa90ca351

from __future__ import annotations
from .basemodelcard import BaseModelCard, BaseModelCardTypedDict
from .ftmodelcard import FTModelCard, FTModelCardTypedDict
from functools import partial
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict
from pydantic.functional_validators import BeforeValidator
from typing import Any, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class RetrieveModelV1ModelsModelIDGetRequestTypedDict(TypedDict):
    model_id: str
    r"""The ID of the model to retrieve."""


class RetrieveModelV1ModelsModelIDGetRequest(BaseModel):
    model_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the model to retrieve."""


ResponseRetrieveModelV1ModelsModelIDGetTypedDict = TypeAliasType(
    "ResponseRetrieveModelV1ModelsModelIDGetTypedDict",
    Union[BaseModelCardTypedDict, FTModelCardTypedDict],
)
r"""Successful Response"""


class UnknownResponseRetrieveModelV1ModelsModelIDGet(BaseModel):
    r"""A ResponseRetrieveModelV1ModelsModelIDGet variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_RESPONSE_RETRIEVE_MODEL_V1_MODELS_MODEL_ID_GET_VARIANTS: dict[str, Any] = {
    "base": BaseModelCard,
    "fine-tuned": FTModelCard,
}


ResponseRetrieveModelV1ModelsModelIDGet = Annotated[
    Union[BaseModelCard, FTModelCard, UnknownResponseRetrieveModelV1ModelsModelIDGet],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_RESPONSE_RETRIEVE_MODEL_V1_MODELS_MODEL_ID_GET_VARIANTS,
            unknown_cls=UnknownResponseRetrieveModelV1ModelsModelIDGet,
            union_name="ResponseRetrieveModelV1ModelsModelIDGet",
        )
    ),
]
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d6adc687c2d9

from __future__ import annotations
from .judgeclassificationoutput import (
    JudgeClassificationOutput,
    JudgeClassificationOutputTypedDict,
)
from .judgeregressionoutput import JudgeRegressionOutput, JudgeRegressionOutputTypedDict
from datetime import datetime
from functools import partial
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict, model_serializer
from pydantic.functional_validators import BeforeValidator
from typing import Any, List, Literal, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


JudgeOutputUnionTypedDict = TypeAliasType(
    "JudgeOutputUnionTypedDict",
    Union[JudgeClassificationOutputTypedDict, JudgeRegressionOutputTypedDict],
)


class UnknownJudgeOutputUnion(BaseModel):
    r"""A JudgeOutputUnion variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_JUDGE_OUTPUT_UNION_VARIANTS: dict[str, Any] = {
    "CLASSIFICATION": JudgeClassificationOutput,
    "REGRESSION": JudgeRegressionOutput,
}


JudgeOutputUnion = Annotated[
    Union[JudgeClassificationOutput, JudgeRegressionOutput, UnknownJudgeOutputUnion],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_JUDGE_OUTPUT_UNION_VARIANTS,
            unknown_cls=UnknownJudgeOutputUnion,
            union_name="JudgeOutputUnion",
        )
    ),
]


class JudgeTypedDict(TypedDict):
    id: str
    created_at: datetime
    updated_at: datetime
    deleted_at: Nullable[datetime]
    owner_id: str
    workspace_id: str
    name: str
    description: str
    model_name: str
    output: JudgeOutputUnionTypedDict
    instructions: str
    tools: List[str]
    up_revision: NotRequired[Nullable[str]]
    down_revision: NotRequired[Nullable[str]]
    base_revision: NotRequired[Nullable[str]]


class Judge(BaseModel):
    id: str

    created_at: datetime

    updated_at: datetime

    deleted_at: Nullable[datetime]

    owner_id: str

    workspace_id: str

    name: str

    description: str

    model_name: str

    output: JudgeOutputUnion

    instructions: str

    tools: List[str]

    up_revision: OptionalNullable[str] = UNSET

    down_revision: OptionalNullable[str] = UNSET

    base_revision: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["up_revision", "down_revision", "base_revision"])
        nullable_fields = set(
            ["deleted_at", "up_revision", "down_revision", "base_revision"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 58d633507527

from __future__ import annotations
from .codeinterpretertool import CodeInterpreterTool, CodeInterpreterToolTypedDict
from .completionargs import CompletionArgs, CompletionArgsTypedDict
from .conversationinputs import ConversationInputs, ConversationInputsTypedDict
from .customconnector import CustomConnector, CustomConnectorTypedDict
from .documentlibrarytool import DocumentLibraryTool, DocumentLibraryToolTypedDict
from .functiontool import FunctionTool, FunctionToolTypedDict
from .guardrailconfig import GuardrailConfig, GuardrailConfigTypedDict
from .imagegenerationtool import ImageGenerationTool, ImageGenerationToolTypedDict
from .metadatadict import MetadataDict, MetadataDictTypedDict
from .websearchpremiumtool import WebSearchPremiumTool, WebSearchPremiumToolTypedDict
from .websearchtool import WebSearchTool, WebSearchToolTypedDict
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import Field, model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ConversationStreamRequestHandoffExecution = Literal[
    "client",
    "server",
]


ConversationStreamRequestToolTypedDict = TypeAliasType(
    "ConversationStreamRequestToolTypedDict",
    Union[
        FunctionToolTypedDict,
        WebSearchToolTypedDict,
        WebSearchPremiumToolTypedDict,
        CodeInterpreterToolTypedDict,
        ImageGenerationToolTypedDict,
        DocumentLibraryToolTypedDict,
        CustomConnectorTypedDict,
    ],
)


ConversationStreamRequestTool = Annotated[
    Union[
        CodeInterpreterTool,
        CustomConnector,
        DocumentLibraryTool,
        FunctionTool,
        ImageGenerationTool,
        WebSearchTool,
        WebSearchPremiumTool,
    ],
    Field(discriminator="type"),
]


ConversationStreamRequestAgentVersionTypedDict = TypeAliasType(
    "ConversationStreamRequestAgentVersionTypedDict", Union[str, int]
)


ConversationStreamRequestAgentVersion = TypeAliasType(
    "ConversationStreamRequestAgentVersion", Union[str, int]
)


class ConversationStreamRequestTypedDict(TypedDict):
    inputs: ConversationInputsTypedDict
    stream: Literal[True]
    store: NotRequired[Nullable[bool]]
    handoff_execution: NotRequired[Nullable[ConversationStreamRequestHandoffExecution]]
    instructions: NotRequired[Nullable[str]]
    tools: NotRequired[Nullable[List[ConversationStreamRequestToolTypedDict]]]
    completion_args: NotRequired[Nullable[CompletionArgsTypedDict]]
    guardrails: NotRequired[Nullable[List[GuardrailConfigTypedDict]]]
    name: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[MetadataDictTypedDict]]
    agent_id: NotRequired[Nullable[str]]
    agent_version: NotRequired[Nullable[ConversationStreamRequestAgentVersionTypedDict]]
    model: NotRequired[Nullable[str]]


class ConversationStreamRequest(BaseModel):
    inputs: ConversationInputs

    stream: Annotated[
        Annotated[Optional[Literal[True]], AfterValidator(validate_const(True))],
        pydantic.Field(alias="stream"),
    ] = True

    store: OptionalNullable[bool] = UNSET

    handoff_execution: OptionalNullable[ConversationStreamRequestHandoffExecution] = (
        UNSET
    )

    instructions: OptionalNullable[str] = UNSET

    tools: OptionalNullable[List[ConversationStreamRequestTool]] = UNSET

    completion_args: OptionalNullable[CompletionArgs] = UNSET

    guardrails: OptionalNullable[List[GuardrailConfig]] = UNSET

    name: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET

    metadata: OptionalNullable[MetadataDict] = UNSET

    agent_id: OptionalNullable[str] = UNSET

    agent_version: OptionalNullable[ConversationStreamRequestAgentVersion] = UNSET

    model: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "stream",
                "store",
                "handoff_execution",
                "instructions",
                "tools",
                "completion_args",
                "guardrails",
                "name",
                "description",
                "metadata",
                "agent_id",
                "agent_version",
                "model",
            ]
        )
        nullable_fields = set(
            [
                "store",
                "handoff_execution",
                "instructions",
                "tools",
                "completion_args",
                "guardrails",
                "name",
                "description",
                "metadata",
                "agent_id",
                "agent_version",
                "model",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ConversationStreamRequest.model_rebuild()
except NameError:
    pass



import logging

import httpx
from authlib.integrations.httpx_client import AsyncOAuth2Client as AsyncOAuth2ClientBase
from authlib.oauth2.rfc8414 import AuthorizationServerMetadata

from mistralai.client.types import BaseModel

logger = logging.getLogger(__name__)


class Oauth2AuthorizationScheme(BaseModel):
    """Information about the oauth flow to perform with the authorization server."""

    authorization_url: str
    token_url: str
    scope: list[str]
    description: str | None = None
    refresh_url: str | None = None


class OAuthParams(BaseModel):
    """Required params for authorization."""

    scheme: Oauth2AuthorizationScheme
    client_id: str
    client_secret: str


class AsyncOAuth2Client(AsyncOAuth2ClientBase):
    """Subclass of the Async httpx oauth client which provides a constructor from OAuthParams."""

    @classmethod
    def from_oauth_params(cls, oauth_params: OAuthParams) -> "AsyncOAuth2Client":
        return cls(
            client_id=oauth_params.client_id,
            client_secret=oauth_params.client_secret,
            scope=oauth_params.scheme.scope,
        )


async def get_well_known_authorization_server_metadata(
    server_url: str,
) -> AuthorizationServerMetadata | None:
    """Fetch the metadata from the well-known location.

    This should be available on MCP servers as described by the specification:
    https://modelcontextprotocol.io/specification/2025-03-26/basic/authorization#2-3-server-metadata-discovery.
    """
    well_known_url = f"{server_url}/.well-known/oauth-authorization-server"
    response = await httpx.AsyncClient().get(well_known_url)
    if 200 <= response.status_code < 300:
        try:
            server_metadata = AuthorizationServerMetadata(**response.json())
            server_metadata.validate()
            return server_metadata
        except ValueError:
            logger.exception("Failed to parse oauth well-known metadata")
            return None
    else:
        logger.error(f"Failed to get oauth well-known metadata from {server_url}")
        return None


async def get_oauth_server_metadata(server_url: str) -> AuthorizationServerMetadata:
    """Fetch the metadata from the authorization server to perform the oauth flow."""
    # 1) attempt to get the metadata from the resource server at /.well-known/oauth-protected-resource
    # TODO: new self-discovery protocol, not released yet

    # 2) attempt to get the metadata from the authorization server at /.well-known/oauth-authorization-server
    metadata = await get_well_known_authorization_server_metadata(server_url=server_url)
    if metadata is not None:
        return metadata

    # 3) fallback on default endpoints
    # https://modelcontextprotocol.io/specification/2025-03-26/basic/authorization#2-3-3-fallbacks-for-servers-without-metadata-discovery
    return AuthorizationServerMetadata(
        issuer=server_url,
        authorization_endpoint=f"{server_url}/authorize",
        token_endpoint=f"{server_url}/token",
        register_endpoint=f"{server_url}/register",
        response_types_supported=["code"],
        response_modes_supported=["query"],
        grant_types_supported=["authorization_code", "refresh_token"],
        token_endpoint_auth_methods_supported=["client_secret_basic"],
        code_challenge_methods_supported=["S256", "plain"],
    )


async def dynamic_client_registration(
    register_endpoint: str,
    redirect_url: str,
    async_client: httpx.AsyncClient,
) -> tuple[str, str]:
    """Try to register the client dynamically with an MCP server.

    Returns a client_id and client_secret.
    """
    # Construct the registration request payload
    registration_payload = {
        "client_name": "MistralSDKClient",
        "grant_types": ["authorization_code", "refresh_token"],
        "token_endpoint_auth_method": "client_secret_basic",
        "response_types": ["code"],
        "redirect_uris": [redirect_url],
    }

    # Make the registration request
    response = await async_client.post(register_endpoint, json=registration_payload)
    try:
        response.raise_for_status()
        registration_info = response.json()
        client_id = registration_info["client_id"]
        client_secret = registration_info["client_secret"]
    except Exception as e:
        raise ValueError(
            f"Client registration failed: status={response.status_code}, error={response.text}"
        ) from e
    return client_id, client_secret


async def build_oauth_params(
    server_url: str,
    redirect_url: str,
    client_id: str | None = None,
    client_secret: str | None = None,
    scope: list[str] | None = None,
    async_client: httpx.AsyncClient | None = None,
) -> OAuthParams:
    """Get issuer metadata and build the oauth required params."""
    metadata = await get_oauth_server_metadata(server_url=server_url)
    oauth_scheme = Oauth2AuthorizationScheme(
        authorization_url=metadata.authorization_endpoint,
        token_url=metadata.token_endpoint,
        scope=scope or [],
        refresh_url=metadata.token_endpoint
        if "refresh_token" in metadata.grant_types_supported
        else None,
    )
    if client_id and client_secret:
        return OAuthParams(
            client_id=client_id,
            client_secret=client_secret,
            scheme=oauth_scheme,
        )

    # Try to dynamically register the client
    if async_client:
        reg_client_id, reg_client_secret = await dynamic_client_registration(
            register_endpoint=metadata.registration_endpoint,
            redirect_url=redirect_url,
            async_client=async_client,
        )
    else:
        async with httpx.AsyncClient() as async_client:
            reg_client_id, reg_client_secret = await dynamic_client_registration(
                register_endpoint=metadata.registration_endpoint,
                redirect_url=redirect_url,
                async_client=async_client,
            )
    return OAuthParams(
        client_id=reg_client_id,
        client_secret=reg_client_secret,
        scheme=oauth_scheme,
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6b3ce5be1294

from __future__ import annotations
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ListVoicesV1AudioVoicesGetType = Literal[
    "all",
    "custom",
    "preset",
]
r"""Filter the voices between customs and presets"""


class ListVoicesV1AudioVoicesGetRequestTypedDict(TypedDict):
    limit: NotRequired[int]
    r"""Maximum number of voices to return"""
    offset: NotRequired[int]
    r"""Offset for pagination"""
    type: NotRequired[ListVoicesV1AudioVoicesGetType]
    r"""Filter the voices between customs and presets"""


class ListVoicesV1AudioVoicesGetRequest(BaseModel):
    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Maximum number of voices to return"""

    offset: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0
    r"""Offset for pagination"""

    type: Annotated[
        Optional[ListVoicesV1AudioVoicesGetType],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "all"
    r"""Filter the voices between customs and presets"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["limit", "offset", "type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1733e4765106

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


SSETypes = Union[
    Literal[
        "conversation.response.started",
        "conversation.response.done",
        "conversation.response.error",
        "message.output.delta",
        "tool.execution.started",
        "tool.execution.delta",
        "tool.execution.done",
        "agent.handoff.started",
        "agent.handoff.done",
        "function.call.delta",
    ],
    UnrecognizedStr,
]
r"""Server side events sent when streaming a conversation response."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 17e125b1022c

from __future__ import annotations
from .connector import Connector, ConnectorTypedDict
from .paginationresponse import PaginationResponse, PaginationResponseTypedDict
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class PaginatedConnectorsTypedDict(TypedDict):
    items: List[ConnectorTypedDict]
    pagination: PaginationResponseTypedDict


class PaginatedConnectors(BaseModel):
    items: List[Connector]

    pagination: PaginationResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: a2b468c87a8a

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class AuthURLResponseTypedDict(TypedDict):
    auth_url: str
    ttl: int


class AuthURLResponse(BaseModel):
    auth_url: str

    ttl: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 44727997dacb

from __future__ import annotations
from .agenthandoffentry import AgentHandoffEntry, AgentHandoffEntryTypedDict
from .functioncallentry import FunctionCallEntry, FunctionCallEntryTypedDict
from .functionresultentry import FunctionResultEntry, FunctionResultEntryTypedDict
from .messageinputentry import MessageInputEntry, MessageInputEntryTypedDict
from .messageoutputentry import MessageOutputEntry, MessageOutputEntryTypedDict
from .toolexecutionentry import ToolExecutionEntry, ToolExecutionEntryTypedDict
from typing import Union
from typing_extensions import TypeAliasType


InputEntriesTypedDict = TypeAliasType(
    "InputEntriesTypedDict",
    Union[
        FunctionResultEntryTypedDict,
        MessageInputEntryTypedDict,
        MessageOutputEntryTypedDict,
        AgentHandoffEntryTypedDict,
        ToolExecutionEntryTypedDict,
        FunctionCallEntryTypedDict,
    ],
)


InputEntries = TypeAliasType(
    "InputEntries",
    Union[
        FunctionResultEntry,
        MessageInputEntry,
        MessageOutputEntry,
        AgentHandoffEntry,
        ToolExecutionEntry,
        FunctionCallEntry,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: dbb7fb36a4fd

from __future__ import annotations
from .failure import Failure, FailureTypedDict
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class WorkflowExecutionFailedAttributesTypedDict(TypedDict):
    r"""Attributes for workflow execution failed events."""

    task_id: str
    r"""Unique identifier for the task within the workflow execution."""
    failure: FailureTypedDict
    r"""Represents an error or exception that occurred during execution."""


class WorkflowExecutionFailedAttributes(BaseModel):
    r"""Attributes for workflow execution failed events."""

    task_id: str
    r"""Unique identifier for the task within the workflow execution."""

    failure: Failure
    r"""Represents an error or exception that occurred during execution."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c40066d868c9

from datetime import datetime
import sys


def parse_datetime(datetime_string: str) -> datetime:
    """
    Convert a RFC 3339 / ISO 8601 formatted string into a datetime object.
    Python versions 3.11 and later support parsing RFC 3339 directly with
    datetime.fromisoformat(), but for earlier versions, this function
    encapsulates the necessary extra logic.
    """
    # Python 3.11 and later can parse RFC 3339 directly
    if sys.version_info >= (3, 11):
        return datetime.fromisoformat(datetime_string)
    
    # For Python 3.10 and earlier, a common ValueError is trailing 'Z' suffix,
    # so fix that upfront.
    if datetime_string.endswith("Z"):
        datetime_string = datetime_string[:-1] + "+00:00"

    return datetime.fromisoformat(datetime_string)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0b1bd06b24af

from __future__ import annotations
from .fetchfieldoptioncountsrequest import (
    FetchFieldOptionCountsRequest,
    FetchFieldOptionCountsRequestTypedDict,
)
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class GetChatCompletionFieldOptionsCountsV1ObservabilityChatCompletionFieldsFieldNameOptionsCountsPostRequestTypedDict(
    TypedDict
):
    field_name: str
    fetch_field_option_counts_request: FetchFieldOptionCountsRequestTypedDict


class GetChatCompletionFieldOptionsCountsV1ObservabilityChatCompletionFieldsFieldNameOptionsCountsPostRequest(
    BaseModel
):
    field_name: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    fetch_field_option_counts_request: Annotated[
        FetchFieldOptionCountsRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6685e3b50b50

from __future__ import annotations
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class ConversationUsageInfoTypedDict(TypedDict):
    prompt_tokens: NotRequired[int]
    completion_tokens: NotRequired[int]
    total_tokens: NotRequired[int]
    connector_tokens: NotRequired[Nullable[int]]
    connectors: NotRequired[Nullable[Dict[str, int]]]


class ConversationUsageInfo(BaseModel):
    prompt_tokens: Optional[int] = 0

    completion_tokens: Optional[int] = 0

    total_tokens: Optional[int] = 0

    connector_tokens: OptionalNullable[int] = UNSET

    connectors: OptionalNullable[Dict[str, int]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "prompt_tokens",
                "completion_tokens",
                "total_tokens",
                "connector_tokens",
                "connectors",
            ]
        )
        nullable_fields = set(["connector_tokens", "connectors"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


ResponseFormats = Literal[
    "text",
    "json_object",
    "json_schema",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 8d809b14b144

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ImportDatasetFromPlaygroundRequestTypedDict(TypedDict):
    conversation_ids: List[str]


class ImportDatasetFromPlaygroundRequest(BaseModel):
    conversation_ids: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 057aecb07275

from __future__ import annotations
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from .systemmessage import SystemMessage, SystemMessageTypedDict
from .toolmessage import ToolMessage, ToolMessageTypedDict
from .usermessage import UserMessage, UserMessageTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag
from typing import List, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ChatModerationRequestInputs2TypedDict = TypeAliasType(
    "ChatModerationRequestInputs2TypedDict",
    Union[
        SystemMessageTypedDict,
        UserMessageTypedDict,
        AssistantMessageTypedDict,
        ToolMessageTypedDict,
    ],
)


ChatModerationRequestInputs2 = Annotated[
    Union[
        Annotated[AssistantMessage, Tag("assistant")],
        Annotated[SystemMessage, Tag("system")],
        Annotated[ToolMessage, Tag("tool")],
        Annotated[UserMessage, Tag("user")],
    ],
    Discriminator(lambda m: get_discriminator(m, "role", "role")),
]


ChatModerationRequestInputs1TypedDict = TypeAliasType(
    "ChatModerationRequestInputs1TypedDict",
    Union[
        SystemMessageTypedDict,
        UserMessageTypedDict,
        AssistantMessageTypedDict,
        ToolMessageTypedDict,
    ],
)


ChatModerationRequestInputs1 = Annotated[
    Union[
        Annotated[AssistantMessage, Tag("assistant")],
        Annotated[SystemMessage, Tag("system")],
        Annotated[ToolMessage, Tag("tool")],
        Annotated[UserMessage, Tag("user")],
    ],
    Discriminator(lambda m: get_discriminator(m, "role", "role")),
]


ChatModerationRequestInputs3TypedDict = TypeAliasType(
    "ChatModerationRequestInputs3TypedDict",
    Union[
        List[ChatModerationRequestInputs1TypedDict],
        List[List[ChatModerationRequestInputs2TypedDict]],
    ],
)
r"""Chat to classify"""


ChatModerationRequestInputs3 = TypeAliasType(
    "ChatModerationRequestInputs3",
    Union[List[ChatModerationRequestInputs1], List[List[ChatModerationRequestInputs2]]],
)
r"""Chat to classify"""


class ChatModerationRequestTypedDict(TypedDict):
    inputs: ChatModerationRequestInputs3TypedDict
    r"""Chat to classify"""
    model: str


class ChatModerationRequest(BaseModel):
    inputs: Annotated[ChatModerationRequestInputs3, pydantic.Field(alias="input")]
    r"""Chat to classify"""

    model: str


try:
    ChatModerationRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f326afe63958

from __future__ import annotations
from .scheduleoverlappolicy import ScheduleOverlapPolicy
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SchedulePolicyTypedDict(TypedDict):
    catchup_window_seconds: NotRequired[int]
    r"""After a Temporal server is unavailable, amount of time in seconds in the past to execute missed actions."""
    overlap: NotRequired[ScheduleOverlapPolicy]
    r"""Controls what happens when a workflow would be started by a schedule but
    one is already running.
    """
    pause_on_failure: NotRequired[bool]
    r"""Whether to pause the schedule after a workflow failure."""


class SchedulePolicy(BaseModel):
    catchup_window_seconds: Optional[int] = 31536000
    r"""After a Temporal server is unavailable, amount of time in seconds in the past to execute missed actions."""

    overlap: Optional[ScheduleOverlapPolicy] = None
    r"""Controls what happens when a workflow would be started by a schedule but
    one is already running.
    """

    pause_on_failure: Optional[bool] = False
    r"""Whether to pause the schedule after a workflow failure."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["catchup_window_seconds", "overlap", "pause_on_failure"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1f0b404ba621

from __future__ import annotations
from .deploymentresponse import DeploymentResponse, DeploymentResponseTypedDict
from mistralai.client.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DeploymentListResponseTypedDict(TypedDict):
    deployments: List[DeploymentResponseTypedDict]
    r"""List of deployments"""


class DeploymentListResponse(BaseModel):
    deployments: List[DeploymentResponse]
    r"""List of deployments"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 16c54f54e60f

from __future__ import annotations
from .signalinvocationbody import SignalInvocationBody, SignalInvocationBodyTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class SignalWorkflowExecutionV1WorkflowsExecutionsExecutionIDSignalsPostRequestTypedDict(
    TypedDict
):
    execution_id: str
    signal_invocation_body: SignalInvocationBodyTypedDict


class SignalWorkflowExecutionV1WorkflowsExecutionsExecutionIDSignalsPostRequest(
    BaseModel
):
    execution_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    signal_invocation_body: Annotated[
        SignalInvocationBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
